/**
 * @package css-modules-types-loader
 * @license MIT
 * @version 0.5.7
 * @author nuintun <nuintun@qq.com>
 * @description A webpack loader that dynamically generates TypeScript typings for CSS modules from css-loader.
 * @see https://github.com/nuintun/css-modules-types-loader#readme
 */

import { schema } from './schema.js';
import { generate, terminate } from './generate/index.js';

/**
 * @module index
 */
const initialized = new Set();
const loader = function loader(content, sourceMap, additionalData) {
  const callback = this.async();
  const logger = this.getLogger('css-modules-types-loader');
  const options = this.getOptions(schema);
  const { _compiler: compiler } = this;
  if (compiler && !initialized.has(compiler)) {
    initialized.add(compiler);
    const teardown = () => {
      terminate().catch(error => {
        logger.error(error.message);
      });
      initialized.delete(compiler);
    };
    if (compiler.watchMode) {
      compiler.hooks.watchClose.tap('css-modules-types-loader', teardown);
    } else {
      compiler.hooks.done.tap('css-modules-types-loader', teardown);
    }
  }
  callback(null, content, sourceMap, additionalData);
  generate(`${this.resourcePath}.d.ts`, content, options).catch(error => {
    logger.error(error.message);
  });
};

export { loader as default };
